local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end

local AddonName, AddonTable = ...
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary
AddonTable.Spells = {}
local spells = AddonTable.Spells

-------------------------------------- Begin code yanked from BloodShieldTracker
local TipFrame = CreateFrame(
	"GameTooltip", "BST_Tooltip", nil, "GameTooltipTemplate")
TipFrame:SetOwner(UIParent, "ANCHOR_NONE")
local function GetNumericValue(...)
    local value
    local valueText
    local region
    local regionText

    for i = 1, select("#", ...) do
        region = select(i, ...)
        if region and region:GetObjectType() == "FontString" then
            regionText = region:GetText()
            if regionText then
                valueText = regionText:match("%d+")
                if valueText then
                    value = tonumber(valueText)
                    if value then
                        return value
                    end
                end
            end
        end
    end
end
local function GetShieldAmount()
	TipFrame:ClearLines()
	TipFrame:SetUnitBuff("player", "Blood Shield")
	amount = GetNumericValue(TipFrame:GetRegions())
	if amount then
		return amount
	else
		return 0
	end
end
---------------------------------------- End code yanked from BloodShieldTracker

local function getRuneCdUp(runeId)
	start,duration,t = GetRuneCooldown(runeId)
	if start == 0 then
		return GetTime()
	else
		return start + duration
	end
end

local function getRuneCd(runeId)
	return getRuneCdUp(runeId) - c.getGcdUp()
end

local function isFullyDepleted(runeId)
	gcdUp = c.getGcdUp()
	start,duration,t = GetRuneCooldown(runeId)
	return start > gcdUp
end

local function runeAvailable(runeId, delay)
	if delay == nil then
		delay = 0
	end
	return getRuneCdUp(runeId) <= c.getGcdUp() + delay
end

local function isDeathRune(runeId)
	return GetRuneType(runeId) == 4
end

function spells.bothRunesAvailable(runeName, delay)
	if runeName == "Blood" then
		return runeAvailable(1, delay) and runeAvailable(2, delay)
	end
	if runeName == "Unholy" then
		return runeAvailable(3, delay) and runeAvailable(4, delay)
	end
	if runeName == "Frost" then
		return runeAvailable(5, delay) and runeAvailable(6, delay)
	end
	print("Bad rune name: ", runeName)
end

local function noCDsUp()
	return not s.Buff("Bone Shield", "player")
		and not s.Buff("Vampiric Blood", "player")
		and not s.Buff("Dancing Rune Weapon", "player")
		and not s.Buff("Icebound Fortitude", "player")
end

local function shouldRuneTap()
	return s.HealthPercent("player") <= 90
end

local function shouldDeathStrikeForHealth()
	return s.HealthPercent("player") <= 93
end

local function shieldIsUncapped()
	return GetShieldAmount() <= s.Health("player") * .93
end

local function canBloodTapForDeathStrike()
	local frost = 0
	local unholy = 0
	local death = 0
	if runeAvailable(3) then
		if isDeathRune(3) then
			death = death + 1
		else
			unholy = unholy + 1
		end
	end
	if runeAvailable(4) then
		if isDeathRune(4) then
			death = death + 1
		else
			unholy = unholy + 1
		end
	end
	if runeAvailable(5) then
		if isDeathRune(5) then
			death = death + 1
		else
			frost = frost + 1
		end
	end
	if runeAvailable(6) then
		if isDeathRune(6) then
			death = death + 1
		else
			frost = frost + 1
		end
	end
	return (frost + unholy + death == 1)
		or (death == 0 and (
				(frost == 0 and unholy > 0)
			or (unholy == 0 and frost > 0)))
end

local function hasBothDiseases()
	return c.hasMyDebuff("Frost Fever") and c.hasMyDebuff("Blood Plague")
end

------------------------------------------------------------------------- common
spells["Horn of Winter"] = {
	SpellID = 57330,
	CastTime = 0,
	ExtraCheck = function()
		return s.MaxPower("player") - s.Power("player") >= 10
			and c.getCooldown("Horn of Winter") == 0 -- why is this necessary??
	end
}

spells["Empower Rune Weapon"] = {
	SpellID = 47568,
	NoGCD = true,
}

spells["Empower Rune Weapon when low"] = {
	SpellID = 47568,
	NoGCD = true,
	ExtraCheck = function()
		return not runeAvailable(1)
			and not runeAvailable(2)
			and not runeAvailable(3)
			and not runeAvailable(4)
			and not runeAvailable(5)
			and not runeAvailable(6)
			and s.Power("player") < 32
	end
}

spells["Outbreak"] = {
	SpellID = 77575,
	ExtraCheck = function()
		return not hasBothDiseases()
			and c.getCooldown("Outbreak") == 0 -- why is this necessary??
	end
}

spells["Death and Decay"] = {
	EnemyTargetNeeded = 1,
}

spells["Death and Decay UU"] = {
	SpellID = 43265,
	ExtraCheck = function()
		return bothRunesAvailable("Unholy")
	end
}

-------------------------------------------------------------------------- blood
spells["Outbreak for Debuffs"] = {
	SpellID = 77575,
	CastTime = 0,
	ExtraCheck = function()
		return not c.isSlowed() or not c.isWeakened()
	end
}

spells["Icy Touch for Debuff"] = {
	SpellID = 45477,
	Debuff = c.SLOW_MELEE_DEBUFFS,
	ExtraCheck = function()
		return c.getCooldown("Outbreak") > 3
	end
}

spells["Plague Strike for Debuff"] = {
	SpellID = 45462,
	Debuff = c.WEAKEN_DEBUFFS,
	ExtraCheck = function()
		return c.getCooldown("Outbreak") > 3
	end
}

-- Currently I assume it's worth heart striking for RP.  If that turns out to
-- be worthless, we should only heart strike when we're sure it won't prevent
-- a well-times rune tap.
spells["Heart Strike"] = {
	SpellID = 55050,
	ExtraCheck = function()
		if runeAvailable(1) then
			return not isDeathRune(1) and runeAvailable(2, 1.5)
		end
		return runeAvailable(2) and not isDeathRune(2) and runeAvailable(1, 1.5)
	end
}

spells["Death Strike"] = {
	SpellID = 49998,
}

spells["Death Strike for Health"] = {
	SpellID = 49998,
	ExtraCheck = function()
		return shouldDeathStrikeForHealth()
	end
}

spells["Death Strike for Runic Empowerment"] = {
	SpellID = 49998,
	ExtraCheck = function()
		return not ( -- keep at least one rune fully depleted
			isFullyDepleted(3) 
			or isFullyDepleted(4) 
			or isFullyDepleted(5) 
			or isFullyDepleted(6))
	end
}

spells["Death Strike for Rune Cooldown"] = {
	SpellID = 49998,
	ExtraCheck = function()
		return bothRunesAvailable("Frost") or bothRunesAvailable("Unholy")
	end
}

spells["Death Strike for Shield"] = {
	SpellID = 49998,
	ExtraCheck = function()
		return shieldIsUncapped()
	end
}

spells["Death Strike to Save Shield"] = {
	SpellID = 49998,
	Buff = "Blood Shield",
	BuffUnit = "player",
	EarlyRefresh = 2.5,
	-- ExtraCheck = function()
		-- t,t,t,t,t,t,expires = UnitBuff("player", "Blood Shield")
		-- return expires and expires - c.getGcdUp() < 2.5
	-- end
}

spells["Death Strike if Two Available"] = {
	SpellID = 49998,
	ExtraCheck = function()
		avail = 0
		if runeAvailable(3, 1.5) then
			avail = avail + 1
		end
		if runeAvailable(4, 1.5) then
			avail = avail + 1
		end
		if runeAvailable(5, 1.5) then
			avail = avail + 1
		end
		if runeAvailable(6, 1.5) then
			avail = avail + 1
		end
		return avail >= 4
	end
}

spells["Rune Strike for Runic Empowerment"] = {
	SpellID = 56815,
	ExtraCheck = function()
		return isFullyDepleted(3)
			or isFullyDepleted(4)
			or isFullyDepleted(5)
			or isFullyDepleted(6)
	end
}

spells["Rune Strike if Capped"] = {
	SpellID = 56815,
	ExtraCheck = function()
		return s.MaxPower("player") - s.Power("player") < 10
	end
}

spells["Rune Tap"] = {
	SpellID = 48982,
	CastTime = 0,
	NoGCD = true,
	ExtraCheck = function()
		return shouldRuneTap()
	end
}

spells["Bone Shield"] = {
	SpellID = 49222,
	Buff = "Bone Shield",
	BuffUnit = "player",
}

spells["Dancing Rune Weapon"] = {
	SpellID = 49028,
}

spells["Vampiric Blood"] = {
	SpellID = 55233,
	NoGCD = true,
	ExtraCheck = function()
		return s.HealthPercent("player") < 90.2
	end
}

spells["Icebound Fortitude"] = {
	SpellID = 48792,
	NoGCD = true,
}

spells["Fire of the Deep"] = {
	ItemID = 77117,
	Buff = "Elusive",
}

spells["Blood Tap for Death Strike for Health"] = {
	SpellID = 45529,
	NoGCD = true,
	ExtraCheck = function()
		return shouldDeathStrikeForHealth() and canBloodTapForDeathStrike()
	end
}

spells["Blood Tap for Death Strike for Shield"] = {
	SpellID = 45529,
	NoGCD = true,
	ExtraCheck = function()
		return shieldIsUncapped() and canBloodTapForDeathStrike()
	end
}

spells["Free Blood Boil"] = {
	SpellID = 48721,
	ExtraCheck = function()
		return s.MaxPower("player") - s.Power("player") >= 10
			and s.Buff("Crimson Scourge", "player")
	end
}

-------------------------------------------------------------------------- frost
spells["FS"] = {
	SpellID = 49143,
}

spells["Rime"] = {
	SpellID = 49184,
	ExtraCheck = function()
		return s.Buff("Freezing Fog", "player")
	end
}

spells["Rime for FF"] = {
	SpellID = 49184,
	MyDebuff = "Frost Fever",
	ExtraCheck = function()
		return s.Buff("Freezing Fog", "player")
	end
}

spells["HB for FF"] = {
	SpellID = 49184,
	MyDebuff = "Frost Fever",
}

spells["PS for BP"] = {
	SpellID = 45462,
	MyDebuff = "Blood Plague",
	ExtraCheck = function()
		return c.getCooldown("Outbreak") > 3
	end
}

spells["OB"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return hasBothDiseases()
	end
}

spells["OB no Rime"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return not s.Buff("Freezing Fog", "player")
			and hasBothDiseases()
	end
}

spells["OB UU"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return bothRunesAvailable("Unholy", 1)
			and hasBothDiseases()
	end
}

spells["OB BBFF"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return bothRunesAvailable("Blood", 1) 
			and bothRunesAvailable("Frost", 1)
			and hasBothDiseases()
	end
}

spells["HB"] = {
	SpellID = 49184,
}






spells["Frost Presence"] = {
	SpellID = 48266,
	NotIfActive = 1,
}

--spells["Howling Blast"] = spells["HB"]

--spells["Howling Blast for Frost Fever"] = spells["HB for FF"]

--spells["Howling Blast on Freezing Fog"] = spells["Rime"]

spells["Howling Blast under 90"] = {
	SpellID = 49184,
	ExtraCheck = function()
		return s.Power("player") < 90
	end
}

spells["Howling Blast under 60"] = {
	SpellID = 49184,
	ExtraCheck = function()
		return s.Power("player") < 60
	end
}

spells["Howling Blast FF DD"] = {
	SpellID = 49184,
	ExtraCheck = function()
		return bothRunesAvailable("Frost") or bothRunesAvailable("Blood")
	end
}

spells["Plague Strike"] = {
	SpellID = 45462,
}

--spells["Plague Strike for Blood Plague"] = spells["PS for BP"]

spells["Plague Strike UU"] = {
	SpellID = 45462,
	ExtraCheck = function()
		return bothRunesAvailable("Unholy")
	end
}

--spells["Obliterate"] = spells["OB"]

spells["Obliterate KM"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return s.Buff("Killing Machine", "player")
			and hasBothDiseases()
	end
}

spells["Obliterate KM U"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return s.Buff("Killing Machine", "player")
			and (runeAvailable(3) or runeAvailable(4))
			and hasBothDiseases()
	end
}

spells["Obliterate U"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return (runeAvailable(3) or runeAvailable(4))
			and hasBothDiseases()
	end
}

--spells["Obliterate UU next"] = spells["OB UU"]

spells["Obliterate for Rune Cooldowns"] = {
	SpellID = 49020,
	ExtraCheck = function()
		return (bothRunesAvailable("Blood") 
				or (bothRunesAvailable("Frost") 
					and bothRunesAvailable("Unholy")))
			and hasBothDiseases()
	end
}

--spells["Frost Strike"] = spells["FS"]

spells["Frost Strike KM"] = {
	SpellID = 49143,
	ExtraCheck = function()
		return s.Buff("Killing Machine", "player")
	end
}

spells["Frost Strike KM no OB"] = {
	SpellID = 49143,
	ExtraCheck = function()
		return s.Buff("Killing Machine", "player")
			and c.getCooldown("Obliterate") > 1
	end
}

spells["Frost Strike preserve KM"] = {
	SpellID = 49143,
	ExtraCheck = function()
		return not s.Buff("Killing Machine", "player")
			or c.getCooldown("Obliterate") > 1
	end
}

spells["Frost Strike if Capped"] = {
	SpellID = 49143,
	ExtraCheck = function()
		return s.MaxPower("player") - s.Power("player") <= 30
	end
}

spells["Frost Strike over 90"] = {
	SpellID = 49143,
	ExtraCheck = function()
		return s.Power("player") > 90
	end
}

spells["Frost Strike if missing disease"] = {
	SpellID = 49143,
	ExtraCheck = function()
		return not hasBothDiseases()
	end
}

spells["Pillar of Frost"] = {
	SpellID = 51271,
	NoGCD = true,
	CastTime = 0,
}

spells["Raise Dead"] = {
	SpellID = 46584,
}

spells["Blood Tap"] = {
	SpellID = 45529,
	NoGCD = true,
	ExtraCheck = function()
		return isFullyDepleted(1)
	end
}

--------------------------------------------------------------------------- misc
spells["Unholy Presence"] = {
	SpellID = 48265,
	NotIfActive = 1,
}

spells["Icy Touch"] = {
	SpellID = 45477,
	MyDebuff = "Frost Fever",
	EarlyRefresh = 2,
}
